@@ -11,6 +11,7 @@ import Data.Typeable
 import Data.Version
 import Info
 import Lang
+import Flag
 import Lint
 import List
 import Paths_ghc_mod
@@ -31,27 +32,21 @@ usage =    "ghc-mod version " ++ showVersion version ++ "\n"
         ++ "Usage:\n"
         ++ "\t ghc-mod list" ++ ghcOptHelp ++ "[-l]\n"
         ++ "\t ghc-mod lang [-l]\n"
+        ++ "\t ghc-mod flag [-l]\n"
         ++ "\t ghc-mod browse" ++ ghcOptHelp ++ "[-l] [-o] <module> [<module> ...]\n"
         ++ "\t ghc-mod check" ++ ghcOptHelp ++ "<HaskellFile>\n"
-        ++ "\t ghc-mod type" ++ ghcOptHelp ++ "<HaskellFile> <module> <expression>\n"
+        ++ "\t ghc-mod expand" ++ ghcOptHelp ++ "<HaskellFile>\n"
         ++ "\t ghc-mod info" ++ ghcOptHelp ++ "<HaskellFile> <module> <expression>\n"
+        ++ "\t ghc-mod type" ++ ghcOptHelp ++ "<HaskellFile> <module> <line-no> <column-no>\n"
         ++ "\t ghc-mod lint [-h opt] <HaskellFile>\n"
         ++ "\t ghc-mod boot\n"
         ++ "\t ghc-mod help\n"
 
 ----------------------------------------------------------------
 
-defaultOptions :: Options
-defaultOptions = Options {
-    convert = toPlain
-  , hlintOpts = []
-  , ghcOpts = []
-  , operators = False
-  }
-
 argspec :: [OptDescr (Options -> Options)]
 argspec = [ Option "l" ["tolisp"]
-            (NoArg (\opts -> opts { convert = toLisp }))
+            (NoArg (\opts -> opts { outputStyle = LispStyle }))
             "print as a list of Lisp"
           , Option "h" ["hlintOpt"]
             (ReqArg (\h opts -> opts { hlintOpts = h : hlintOpts opts }) "hlintOpt")
@@ -89,15 +84,19 @@ main = flip catches handlers $ do
       "browse" -> concat <$> mapM (browseModule opt) (tail cmdArg)
       "list"   -> listModules opt
       "check"  -> withFile (checkSyntax opt) (safelist cmdArg 1)
-      "type"   -> withFile (typeExpr opt (safelist cmdArg 2) (safelist cmdArg 3)) (safelist cmdArg 1)
+      "expand" -> withFile (checkSyntax opt { expandSplice = True })
+                           (safelist cmdArg 1)
+      "type"  -> withFile (typeExpr opt (safelist cmdArg 2) (read $ safelist cmdArg 3) (read $ safelist cmdArg 4)) (safelist cmdArg 1)
       "info"   -> withFile (infoExpr opt (safelist cmdArg 2) (safelist cmdArg 3)) (safelist cmdArg 1)
       "lint"   -> withFile (lintSyntax opt)  (safelist cmdArg 1)
       "lang"   -> listLanguages opt
+      "flag"   -> listFlags opt
       "boot"   -> do
          mods  <- listModules opt
          langs <- listLanguages opt
-         pre   <- browseModule opt "Prelude"
-         return $ mods ++ langs ++ pre
+         flags <- listFlags opt
+         pre   <- concat <$> mapM (browseModule opt) preBrowsedModules
+         return $ mods ++ langs ++ flags ++ pre
       cmd      -> throw (NoSuchCommand cmd)
     putStr res
   where
@@ -126,11 +125,15 @@ main = flip catches handlers $ do
       | otherwise = xs !! idx
 
 ----------------------------------------------------------------
-toLisp :: [String] -> String
-toLisp ms = "(" ++ unwords quoted ++ ")\n"
-    where
-      quote x = "\"" ++ x ++ "\""
-      quoted = map quote ms
 
-toPlain :: [String] -> String
-toPlain = unlines
+preBrowsedModules :: [String]
+preBrowsedModules = [
+    "Prelude"
+  , "Control.Applicative"
+  , "Control.Monad"
+  , "Control.Exception"
+  , "Data.Char"
+  , "Data.List"
+  , "Data.Maybe"
+  , "System.IO"
+  ]
