@@ -48,21 +48,45 @@ unloaded modules are loaded")
 (defvar ghc-module-names nil)   ;; completion for "import"
 (defvar ghc-merged-keyword nil) ;; completion for type/func/...
 (defvar ghc-language-extensions nil)
+(defvar ghc-option-flags nil)
+(defvar ghc-pragma-names '("LANGUAGE" "OPTIONS_GHC"))
 
 (defconst ghc-keyword-prefix "ghc-keyword-")
 (defvar ghc-keyword-Prelude nil)
+(defvar ghc-keyword-Control.Applicative nil)
+(defvar ghc-keyword-Control.Monad nil)
+(defvar ghc-keyword-Control.Exception nil)
+(defvar ghc-keyword-Data.Char nil)
+(defvar ghc-keyword-Data.List nil)
+(defvar ghc-keyword-Data.Maybe nil)
+(defvar ghc-keyword-System.IO nil)
+
 (defvar ghc-loaded-module nil)
 
 (defun ghc-comp-init ()
   (let* ((syms '(ghc-module-names
-		ghc-language-extensions
-		ghc-keyword-Prelude))
+		 ghc-language-extensions
+		 ghc-option-flags
+		 ghc-keyword-Prelude
+		 ghc-keyword-Control.Applicative
+		 ghc-keyword-Control.Monad
+		 ghc-keyword-Control.Exception
+		 ghc-keyword-Data.Char
+		 ghc-keyword-Data.List
+		 ghc-keyword-Data.Maybe
+		 ghc-keyword-System.IO))
 	 (vals (ghc-boot (length syms))))
     (ghc-set syms vals))
   (ghc-add ghc-module-names "qualified")
   (ghc-add ghc-module-names "hiding")
-  (ghc-add ghc-language-extensions "LANGUAGE")
-  (ghc-merge-keywords '("Prelude"))
+  (ghc-merge-keywords '("Prelude"
+			"Control.Applicative"
+			"Control.Monad"
+			"Control.Exception"
+			"Data.Char"
+			"Data.List"
+			"Data.Maybe"
+			"System.IO"))
   (run-with-idle-timer ghc-idle-timer-interval 'repeat 'ghc-idle-timer))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -172,8 +196,16 @@ unloaded modules are loaded")
     ghc-module-names)
    ((save-excursion
       (beginning-of-line)
-      (looking-at "{-#"))
+      (looking-at "{-# LANGUAGE "))
     ghc-language-extensions)
+   ((save-excursion
+      (beginning-of-line)
+      (looking-at "{-# OPTIONS_GHC "))
+    ghc-option-flags)
+   ((save-excursion
+      (beginning-of-line)
+      (looking-at "{-# "))
+    ghc-pragma-names)
    ((or (bolp)
 	(let ((end (point)))
 	  (save-excursion
@@ -237,13 +269,19 @@ unloaded modules are loaded")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(ghc-defstruct buffer name file)
+
+(defun ghc-buffer-name-file (buf)
+  (ghc-make-buffer (buffer-name buf) (buffer-file-name buf)))
+
 (defun ghc-gather-import-modules-all-buffers ()
-  (let ((bufs (mapcar 'buffer-name (buffer-list)))
-	ret)
+  (let ((bufs (mapcar 'ghc-buffer-name-file (buffer-list)))
+	ret file)
     (save-excursion
       (dolist (buf bufs (ghc-uniq-lol ret))
-	(when (string-match "\\.hs$" buf)
-	  (set-buffer buf)
+	(setq file (ghc-buffer-get-file buf))
+	(when (and file (string-match "\\.hs$" file))
+	  (set-buffer (ghc-buffer-get-name buf))
 	  (ghc-add ret (ghc-gather-import-modules-buffer)))))))
 
 (defun ghc-gather-import-modules-buffer ()
